@@ -1,6 +1,7 @@
 #!/usr/bin/python
 """ efi variables """
 
+import re
 import struct
 import logging
 import datetime
@@ -30,6 +31,11 @@ EFI_VARIABLE_DEFAULT = (EFI_VARIABLE_NON_VOLATILE |
                         EFI_VARIABLE_BOOTSERVICE_ACCESS)
 
 efivar_defaults = {
+    'SecureBoot' : {
+        'attr' : (EFI_VARIABLE_BOOTSERVICE_ACCESS |
+                  EFI_VARIABLE_RUNTIME_ACCESS),
+        'guid' : guids.EfiGlobalVariable,
+    },
     'SecureBootEnable' : {
         'attr' : (EFI_VARIABLE_NON_VOLATILE |
                   EFI_VARIABLE_BOOTSERVICE_ACCESS),
@@ -73,6 +79,37 @@ efivar_defaults = {
                   EFI_VARIABLE_BOOTSERVICE_ACCESS),
         'guid' : guids.Shim,
     },
+    'MokListX' : {
+        'attr' : (EFI_VARIABLE_NON_VOLATILE |
+                  EFI_VARIABLE_BOOTSERVICE_ACCESS),
+        'guid' : guids.Shim,
+    },
+    'MokListTrusted' : {
+        'attr' : (EFI_VARIABLE_BOOTSERVICE_ACCESS |
+                  EFI_VARIABLE_RUNTIME_ACCESS),
+        'guid' : guids.Shim,
+    },
+    'SHIM_DEBUG' : {
+        'attr' : (EFI_VARIABLE_NON_VOLATILE |
+                  EFI_VARIABLE_BOOTSERVICE_ACCESS),
+        'guid' : guids.Shim,
+    },
+    'SHIM_VERBOSE' : {
+        'attr' : (EFI_VARIABLE_NON_VOLATILE |
+                  EFI_VARIABLE_BOOTSERVICE_ACCESS),
+        'guid' : guids.Shim,
+    },
+    'FALLBACK_VERBOSE' : {
+        'attr' : (EFI_VARIABLE_NON_VOLATILE |
+                  EFI_VARIABLE_BOOTSERVICE_ACCESS),
+        'guid' : guids.Shim,
+    },
+    'FB_NO_REBOOT' : {
+        'attr' : (EFI_VARIABLE_NON_VOLATILE |
+                  EFI_VARIABLE_BOOTSERVICE_ACCESS |
+                  EFI_VARIABLE_RUNTIME_ACCESS),
+        'guid' : guids.Shim,
+    },
 }
 
 boot_defaults = {
@@ -82,9 +119,9 @@ boot_defaults = {
     'guid' : guids.EfiGlobalVariable,
 }
 
-sigdb_names = ("PK", "KEK", "db", "dbx", "MokList", "TlsCaCertificate")
+sigdb_names = ("PK", "KEK", "db", "dbx", "MokList", "MokListX", "TlsCaCertificate")
 bool_names  = ('SecureBootEnable', 'CustomMode')
-ascii_names = ('Lang', 'PlatformLang')
+ascii_names = ('Lang', 'PlatformLang', 'SbatLevel')
 blist_names = ('BootOrder', 'BootNext')
 dpath_names = ('ConIn', 'ConOut', 'ErrOut')
 
@@ -178,6 +215,11 @@ class EfiVar:
         self.data = bytes(self.sigdb)
         self.update_time(initial_ts)
 
+    def sigdb_set(self, sigdb):
+        self.sigdb = sigdb
+        self.data = bytes(self.sigdb)
+        self.update_time(self.sigdb.get_cert_ts())
+
     def sigdb_add_cert(self, guid, filename):
         if self.sigdb is None:
             raise RuntimeError
@@ -206,6 +248,10 @@ class EfiVar:
             self.data = b'\x00'
         self.update_time()
 
+    def set_uint32(self, value):
+        self.data = value.to_bytes(4, byteorder = 'little')
+        self.update_time()
+
     def set_boot_entry(self, attr, title, path, optdata = None):
         pathdata = bytes(path)
         self.data = struct.pack('=LH', attr, len(pathdata))
@@ -230,6 +276,7 @@ class EfiVar:
 
     def fmt_ascii(self):
         string = self.data.decode().rstrip('\0')
+        string = re.sub(r'\n', r'\\n', string)
         return f'ascii: "{string}"'
 
     def fmt_boot_entry(self):
@@ -284,6 +331,7 @@ class EfiVar:
 
         return None
 
+# pylint: disable=too-many-public-methods
 class EfiVarList(collections.UserDict):
     """  class for efi variable list """
 
@@ -307,6 +355,13 @@ class EfiVarList(collections.UserDict):
         logging.info('set variable %s: %s', name, value)
         var.set_bool(value)
 
+    def set_uint32(self, name, value):
+        var = self.get(name)
+        if not var:
+            var = self.create(name)
+        logging.info('set variable %s: %s', name, value)
+        var.set_uint32(value)
+
 
     def set_boot_entry(self, index, title, path, optdata = None):
         name = f'Boot{index:04X}'
@@ -385,6 +440,10 @@ class EfiVarList(collections.UserDict):
     def enroll_platform_redhat(self):
         self.enroll_platform_with_cert(certs.REDHAT_PK)
 
+    def enroll_platform_generate(self, name):
+        pk = certs.pk_generate(cn = name)
+        self.enroll_platform_with_cert(pk.name)
+
     def add_microsoft_keys(self):
         self.add_cert('db', guids.MicrosoftVendor, certs.MS_WIN, False)
         self.add_cert('db', guids.MicrosoftVendor, certs.MS_3RD, False)
